@@ -30,12 +30,13 @@ import numpy as np
 from PIL import Image as PILImage
 
 # Rikai
-from rikai.conf import CONF_RIKAI_IMAGE_DEFAULT_FORMAT, options
+from rikai.conf import options
 from rikai.internal.uri_utils import normalize_uri
 from rikai.io import copy, open_output_stream
-from rikai.mixin import Asset, Displayable, ToDict, ToNumpy, ToPIL
+from rikai.mixin import Asset, Displayable, Drawable, ToDict, ToNumpy, ToPIL
 from rikai.spark.types import ImageType
 from rikai.types.geometry import Box2d
+from rikai.viz import Draw, PILRenderer
 
 __all__ = ["Image"]
 
@@ -103,6 +104,19 @@ class Image(ToNumpy, ToPIL, Asset, Displayable, ToDict):
         with PILImage.fromarray(array, mode=mode) as img:
             return cls.from_pil(img, uri, format=format, **kwargs)
 
+    @staticmethod
+    def read(
+        uri: Union[str, Path],
+    ) -> Image:
+        """Create an embedded image from external URI
+
+        Parameters
+        ----------
+        uri : str or Path
+            The URI pointed to an image.
+        """
+        return Image(uri).to_embedded()
+
     @staticmethod
     def from_pil(
         img: PILImage,
@@ -157,8 +171,21 @@ class Image(ToNumpy, ToPIL, Asset, Displayable, ToDict):
         """
         from IPython.display import Image
 
-        with self.open() as fobj:
-            return Image(fobj.read(), **kwargs)
+        if not self.is_embedded and self.uri.startswith("http"):
+            return Image(url=self.uri, **kwargs)
+        else:
+            # Using Data URIs for Databricks Notebook
+            with self.open() as fobj:
+                data = fobj.read()
+                inferred_format = Image(data).format
+                encoded = base64.b64encode(data).decode("utf-8")
+                url = f"data:image;base64,{encoded}"
+                return Image(
+                    url=url, embed=True, format=inferred_format, **kwargs
+                )
+
+    def draw(self, drawable: Union[Drawable, list[Drawable]]) -> Draw:
+        return ImageDraw(self).draw(drawable)
 
     def __repr__(self) -> str:
         if self.is_embedded:
@@ -186,6 +213,12 @@ class Image(ToNumpy, ToPIL, Asset, Displayable, ToDict):
     def __eq__(self, other) -> bool:
         return isinstance(other, Image) and super().__eq__(other)
 
+    def __or__(self, other: Drawable) -> Draw:
+        """Override ``|`` operator to chain images with
+        visualization components.
+        """
+        return self.draw(other)
+
     def to_pil(self) -> PILImage:
         """Return an PIL image.
 
@@ -206,6 +239,13 @@ class Image(ToNumpy, ToPIL, Asset, Displayable, ToDict):
             return {"data": base64.b64encode(self.data).decode("ascii")}
         return {"uri": self.uri}
 
+    def to_embedded(self) -> Image:
+        """Convert this image into an embedded image."""
+        if self.is_embedded:
+            return self
+        with self.open() as img:
+            return Image(img.read())
+
     def save(self, uri: Union[str, Path]) -> Image:
         """Save the image into a file, specified by the file path or URI.
 
@@ -258,3 +298,22 @@ class Image(ToNumpy, ToPIL, Asset, Displayable, ToDict):
                 with pil_image.crop(bbox) as patch:
                     crops.append(Image.from_pil(patch))
         return crops
+
+
+class ImageDraw(Draw):
+    def __init__(self, img: Image):
+        super().__init__()
+        self.img = img.to_pil()
+
+    def display(self, **kwargs) -> "IPython.display.DisplayObject":
+        if not self.layers:
+            raise ValueError("Can not render empty displayable draw")
+
+        render = PILRenderer(self.img)
+        for layer in self.layers:
+            layer._render(render)
+        return Image.from_pil(render.image)
+
+    def _repr_png_(self):
+        """default visualizer for embedded png"""
+        return self.display()._repr_png_()
